﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using Composite.Data;
using Composite.Data.Plugins.DataProvider;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;

namespace Sample.DataProvider.CultureData
{
    /// <summary>
    /// The data provider that deliver data to Composite C1.
    /// This sample only implement IDataProvider (and not IWritableDataProvider) making it a very simple read only data provider.
    /// The Attribute 'ConfigurationElementType' points to the configuration class for this provider 
    /// and is used when your provider is constructed. See CultureDataProviderBuildup.cs fo more.
    /// </summary>
    [ConfigurationElementType(typeof(CultureDataProviderConfigData))]
    public class CultureDataProvider : IDataProvider
    {
        /// <summary>
        /// When your provider has been constructed it is given a 'DataProviderContext' by Composite C1 - this happen before any querying is done.
        /// In this sample we 'new up' all the data at this point - paying an up-front init price, even though we may never be called.
        /// Consider making enspensive initializations lazy.
        /// </summary>
        public DataProviderContext Context
        {
            set
            {
                var cultureDataItems = from CultureInfo ci in CultureInfo.GetCultures(CultureTypes.AllCultures)
                                       select new CultureDataItem(ci, value);

                this.MyCultureData = cultureDataItems.ToList<ICultureData>();
                this.MyCultureDataQueryable = this.MyCultureData.AsQueryable();
            }
        }

        /// <summary>
        /// Returns a specific data item, given it's data ID. Since we only support one data interface - ICulture, 
        /// and this only use CultureDataItemId as IDataId, we expect only to get a CultureDataItemId passed to this method.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="dataId">Expecting a CultureDataItemId, since we only expose/use that IDataId</param>
        /// <returns>The ICulture data item matching the provided CultureDataItemId</returns>
        public T GetData<T>(IDataId dataId) where T : class, IData
        {
            CultureDataItemId cultureId = (CultureDataItemId)dataId;
            return (T)this.MyCultureDataQueryable.Where(f => f.Name == cultureId.CultureInfo.Name).FirstOrDefault();
        }

        /// <summary>
        /// The query interface itself - we created an IQueryable(ICulture) when this provider got it's Context set so we just return that.
        /// </summary>
        /// <typeparam name="T">Expecting ICulture, since we only expose/use that data interface</typeparam>
        /// <returns></returns>
        public IQueryable<T> GetData<T>() where T : class, Composite.Data.IData
        {
            return (IQueryable<T>)this.MyCultureDataQueryable;
        }

        /// <summary>
        /// The data interfaces we support
        /// </summary>
        /// <returns>Returning just ICultureData, the single data interface we support</returns>
        public IEnumerable<Type> GetSupportedInterfaces()
        {
            yield return typeof(ICultureData);
        }

        /// <summary>
        /// The data we hold
        /// </summary>
        private List<ICultureData> MyCultureData { get; set; }

        /// <summary>
        /// IQueryable for the data we hold.
        /// </summary>
        private IQueryable<ICultureData> MyCultureDataQueryable { get; set; }

    }
}
